//  RTS PLAZA Online
//  Copyright (c) 1997-2006, Technical Center of RTS
//
//  Version 2.0

#ifndef __RTSONL_H__
#define __RTSONL_H__

#ifdef __cplusplus
extern "C" {
#endif

///////////////////////////////////////////////////
//    Messages
//
#define RTSONL_MSG_PING                         8

#define RTSONL_MSG_NETINTEGRITYOK               14
#define RTSONL_MSG_NETINTEGRITYLOST             15

#define RTSONL_MSG_DS_CONNECTED                 16
#define RTSONL_MSG_DS_DISCONNECTED              17

#define RTSONL_MSG_DS_SCHEME                    18
#define RTSONL_MSG_MSG_SCHEME                   19

#define RTSONL_MSG_CONN_ERROR                   20
#define RTSONL_MSG_CONN_CLEAR                   21
#define RTSONL_MSG_CONN_FORMAT                  22
#define RTSONL_MSG_CONN_ADDROW                  23
#define RTSONL_MSG_CONN_UPDATEROW               24
#define RTSONL_MSG_CONN_DELETEROW               25
#define RTSONL_MSG_CONN_SYNCHRONIZED            26
#define RTSONL_MSG_CONN_CONNECTED               27
#define RTSONL_MSG_CONN_DISCONNECTED            28
#define RTSONL_MSG_CONN_REPLBEGIN               29
#define RTSONL_MSG_CONN_REPLEND                 30

#define RTSONL_MSG_LOGINOK                      35
#define RTSONL_MSG_LOGINFAILED                  36

#define RTSONL_MSG_TYMESYNCH                    37

#define RTSONL_MSG_BEGINTRANS                   40
#define RTSONL_MSG_COMMIT                       41
#define RTSONL_MSG_ROLLBACK                     42

// Connection types
#define RTSONL_OPEN_DYNAMIC                     0
#define RTSONL_OPEN_STATIC                      1

// Columns separator
#define RTSONL_DELIMS                           "\001"
#define RTSONL_DELIMC                           '\001'

// Flags
#define RTSONL_FLAGS_ENUM_DIRECTION_FORWARD     0x0000
#define RTSONL_FLAGS_ENUM_DIRECTION_BACKWARD    0x0001

// Flags for rtsonlSetIniPathEx
#define RTSONL_INIT_FLAGS_CHAR						0x0
#define RTSONL_INIT_FLAGS_UNICODE					0x1

///////////////////////////////////////////////////
//    Types
//
typedef void (CALLBACK* RTSONL_CALLBACK) ( UINT msgType, void** params );

typedef struct 
    {
    int     version;
    int     tablesCount;
    } RTSONL_DB_SCHEME;

typedef struct
    {
    char    name[64];
    int     columnsCount;
    } RTSONL_TABLE;

typedef struct
    {
    char    name[64];
    char    format[32];
    } RTSONL_COLUMN;

//         
typedef struct 
{
    int     version;        //   
    int     messagesCount;  // 
} RTSONL_MSG_SCHEME;

//       ,   
typedef struct
{
    char    name[64];       // 
    int     fieldsCount;    // 
} RTSONL_MSG;

//       ,    
typedef struct
{
    char    name[64];   // 
    char    format[32]; // 
} RTSONL_MSG_FIELD;


typedef LONG RTSONL_CONNID;

///////////////////////////////////////////////////
//    Functions
//
void WINAPI rtsonlSetIniPath( const char* iniPath );
void WINAPI rtsonlSetIniPathEx( const char* iniPath, UINT32 flags );
BOOL WINAPI rtsonlIdentify( const char* userName, const char* password, const char* extraPwd );
BOOL WINAPI rtsonlRegisterWnd( HWND hwndCallback, UINT message );
BOOL WINAPI rtsonlRegisterCallback( RTSONL_CALLBACK callback );
void WINAPI rtsonlOpen( void );
void WINAPI rtsonlClose( void );
void WINAPI rtsonlCloseAsynch( void );

//-- Database scheme

// function rtsonlOpenScheme SHOULDN'T be used in callback
const RTSONL_DB_SCHEME* WINAPI rtsonlOpenScheme( void );
BOOL WINAPI rtsonlOpenSchemeAsynch( void );
void WINAPI rtsonlCloseScheme( const RTSONL_DB_SCHEME* scheme );

const RTSONL_TABLE* WINAPI rtsonlGetTable( const RTSONL_DB_SCHEME* scheme, int tableIndex );
const RTSONL_COLUMN* WINAPI rtsonlTableGetColumn( const RTSONL_TABLE* table, int columnIndex );

//-- Message scheme

//Synchronous request of messages scheme   !!! SHOULDN'T be used in callback
// result: null -scheme is not found
const RTSONL_MSG_SCHEME* WINAPI rtsonlOpenMsgScheme ();

//Asynchronous request of messages scheme   !!! MAY be used in callback
//Pointer on structure, describing the messages, comes back by the message RTSONL_MSG_MSG_SCHEME.
// result: TRUE  - request is accepted, waiting message RTSONL_MSG_MSG_SCHEME;
//         FALSE - request is not accepted.
BOOL WINAPI rtsonlOpenMsgSchemeAsynch ();

//Close messages scheme
void WINAPI rtsonlCloseMsgScheme
    (const RTSONL_MSG_SCHEME* scheme);  //messages scheme

//Get message descriptor
// result: null -message is not found
const RTSONL_MSG* WINAPI rtsonlGetMsg
    (const RTSONL_MSG_SCHEME* scheme,       //messages scheme
     int                      msgIndex);    //message index [0, scheme->messagesCount)
//
const RTSONL_MSG* WINAPI rtsonlFindMsg
    (const RTSONL_MSG_SCHEME* scheme,       //messages scheme
     const char*              msgName);     //message name

//Get field descriptor
// result: null -field is not found
const RTSONL_MSG_FIELD* WINAPI rtsonlGetMsgField
    (const RTSONL_MSG* msg,         //message descriptor
     int               fieldIndex); //field index [0, msg->fieldsCount)
//
const RTSONL_MSG_FIELD* WINAPI rtsonlFindMsgField
    (const RTSONL_MSG* msg,         //message descriptor
     const char*       fieldName);  //field name

//-- Receiving data

RTSONL_CONNID WINAPI rtsonlOpenConnection( int type,
                            const char* tableName, const char* columns, 
                            const char* indexColumns, const char* recMin, const char* recMax,
                            int flags);

// for views with orderBy clause
RTSONL_CONNID WINAPI rtsonlOpenConnectionWithOrderBy( int type,
                            const char* tableName, const char* columns, 
                            const char* indexColumns, const char* recMin, const char* recMax,
							const char* orderBy,
                            int flags);


RTSONL_CONNID WINAPI rtsonlGetNextConnectionID ();

RTSONL_CONNID WINAPI rtsonlOpenConnectionEx( int type,
                            const char* tableName, const char* columns, 
                            const char* indexColumns, const char* recMin, const char* recMax,
                            int flags,
                            RTSONL_CONNID userConnID);

// for views with orderBy clause
RTSONL_CONNID WINAPI rtsonlOpenConnectionExWithOrderBy( int type,
                            const char* tableName, const char* columns, 
                            const char* indexColumns, const char* recMin, const char* recMax,
							const char* orderBy,
                            int flags,
                            RTSONL_CONNID userConnID);

void WINAPI rtsonlCloseConnection( RTSONL_CONNID conn );

//-- Crypto

// Init crypto
BOOL WINAPI rtsonlInitCryptoSign
    (const char* key_dir,       // directory with copy of key diskette or diskette itself (A:)
     const char* cid,           // user certificate ID (usually a filename of certificate file)
     const char* from_address); // user e-mail address (contained in the body of certificate)

// Close crypto
void WINAPI rtsonlCloseCryptoSign ();

//-- Send message

//Send synchronous message  !!! SHOULDN'T be used in callback
// result: FALSE -error (see rtsonlGetReplyError)
BOOL WINAPI rtsonlSendMsg
    (const RTSONL_MSG_SCHEME* scheme,  //messages scheme;
     const char* msgName,       //message type name;
     const char* fields,        //fields comma-list;
     const char* fieldsValues); //fields values comma-list, appropriate <fields>.
                                //If in value of a field there is the comma "," or backslash "\",
                                //before it backslash "\" should be put.

//Post asynchronous message without reply !!! SHOULD be used in callback
// result: FALSE -error (see rtsonlGetReplyError)
BOOL WINAPI rtsonlPostMsg
    (const RTSONL_MSG_SCHEME* scheme,  //messages scheme;
     const char* msgName,       //message type name;
     const char* fields,        //fields comma-list;
     const char* fieldsValues,  //fields values comma-list, appropriate <fields>.
                                //If in value of a field there is the comma "," or backslash "\",
                                //before it backslash "\" should be put.
     const char* address);

//Get reply message type name
// result: null -reply is a not found
const char* WINAPI rtsonlGetReplyName ();

//Get reply message field value
// result: null -field is a not found
const char* WINAPI rtsonlGetReplyValue
    (const char* field); //field name

//Get last error string
// result: null -no error
const char* WINAPI rtsonlGetReplyError ();

//--- Utilities

void WINAPI rtsonlStringToDateTime( const char* dtText, SYSTEMTIME* pDateTime );
void WINAPIV rtsonlPrintf( const char* format, ... );

// extern "C"
#ifdef __cplusplus
} 
#endif

#endif // __RTSONL_H__
