// RTS PLAZA
// Copyright (c) 1997-98, Technical Center of RTS
//
// RTS PLAZA Online: Console Sample

#include <windows.h>
#include <stdio.h>
#include <conio.h>

#include "Online/RTSOnl/Include/RTSOnl.h"

HANDLE g_syncEvent;
bool g_stop = false;

BOOL WINAPI CtrlHandler( DWORD dwCtrlType )
{
    g_stop = true;
    return TRUE;
}

void DisplayRows( void** params )
{
    for( int ind = 2; params[ind]; ind++ )
        {
        rtsonlPrintf( "%s; ", params[ind] );
        }
    rtsonlPrintf( "\n" );
}

void CALLBACK OnlineCallback( UINT type, void** params )
{
    switch( type )
        {
        case RTSONL_MSG_PING:
            // TODO: remove comment to display Ping messages
            // rtsonlPrintf( "OnlCon: Ping\n" );
            break;

        case RTSONL_MSG_TYMESYNCH:
            {
            SYSTEMTIME* pTime = (SYSTEMTIME*)params[0];
            rtsonlPrintf( "OnlCon: TimeSync %02d.%02d.%04d %02d:%02d:%02d\n",
                    (int)pTime->wDay, (int)pTime->wMonth, (int)pTime->wYear,
                    (int)pTime->wHour, (int)pTime->wMinute, (int)pTime->wSecond );
            }
            break;

        case RTSONL_MSG_NETINTEGRITYOK:
            rtsonlPrintf( "OnlCon: NetIntegrityOk: %s\n", params[0] );
            break;

        case RTSONL_MSG_NETINTEGRITYLOST:
            rtsonlPrintf( "OnlCon: NetIntegrityLost\n" );
            break;

        case RTSONL_MSG_DS_CONNECTED:
            rtsonlPrintf( "OnlCon: DS Connected\n" );
            ::SetEvent(g_syncEvent);
            break;

        case RTSONL_MSG_DS_DISCONNECTED:
            rtsonlPrintf( "OnlCon: DS Disconnected\n" );
            break;

        ////////////////////////////////////////////////////
        // Connections

        case RTSONL_MSG_CONN_ERROR:
            rtsonlPrintf( "OnlCon: Error conn=%d: %s\n", params[0], params[1] );
            break;

        case RTSONL_MSG_CONN_CLEAR:
            rtsonlPrintf( "OnlCon: Clear conn=%d\n", params[0] );
            break;

        case RTSONL_MSG_CONN_FORMAT:
            rtsonlPrintf( "OnlCon: Format conn=%d: columnsCount=%d columnsFormat='%s'\n",
                params[0], params[1], params[2] );
            break;

        case RTSONL_MSG_CONN_ADDROW:
            rtsonlPrintf( "OnlCon: AddRow conn=%d, rowID=%d\n", params[0], params[1] );
            DisplayRows( params );
            break;

        case RTSONL_MSG_CONN_UPDATEROW:
            rtsonlPrintf( "OnlCon: UpdateRow conn=%d, rowID=%d\n", params[0], params[1] );
            DisplayRows( params );
            break;

        case RTSONL_MSG_CONN_DELETEROW:
            rtsonlPrintf( "OnlCon: DeleteRow conn=%d, rowID=%d\n", params[0], params[1] );
            DisplayRows( params );
            break;

        case RTSONL_MSG_CONN_SYNCHRONIZED:
            rtsonlPrintf( "OnlCon: Sychronized conn=%d\n", params[0] );
            ::SetEvent(g_syncEvent);
            break;

        case RTSONL_MSG_CONN_CONNECTED:
            rtsonlPrintf( "OnlCon: Connected conn=%d\n", params[0] );
            break;

        case RTSONL_MSG_CONN_DISCONNECTED:
            rtsonlPrintf( "OnlCon: Disconnected conn=%d\n", params[0] );
            break;

        case RTSONL_MSG_LOGINOK:
            rtsonlPrintf( "OnlCon: Login Ok, username='%s'\n", params[0] );
            break;
        }
}

void GetPassword( char* buf )
{
    BOOL exit = FALSE;
    for( int i = 0; !exit; )
        {
        int c = _getch();
        switch( c )
            {
            case 0:
                _getch();
                break;

            case '\n':
            case '\r':
                exit = TRUE;
                break;

            case 8:
                if( i > 0 )
                    --i;
                else
                    MessageBeep( MB_OK );
                break;

            default:
                buf[ i++ ] = c;
                break;
            }
        }
    buf[ i ] = 0;
}

void Identify( const char* iniPath )
{
    char    userName[64], password[256];
    char    extraPwd[] = "GUI";

    GetPrivateProfileString( "OnlCon", "user_name", "", userName, sizeof(userName), iniPath );
    GetPrivateProfileString( "OnlCon", "password", "", password, sizeof(password), iniPath );
    if( userName[0] )
        {
        if( strcmp(password, "?") == 0 )
            {
            printf( "enter password for '%s':", userName );
            GetPassword( password );
            }
        rtsonlIdentify( userName, password, extraPwd );
        }
}

void main( int argc, char** argv )
{
    rtsonlPrintf( "RTS PLAZA Online. Console Sample Version 2.0\n"
            "Copyright (c) 1997-98, Technical Center of RTS\n" );

    char* iniPath = ".\\OnlCon.ini";
    if( argc > 1 )
        iniPath = argv[1];
    rtsonlPrintf( "Using ini path: '%s'\n", iniPath );

//    SetConsoleOutputCP( GetACP() );
    SetConsoleCtrlHandler(CtrlHandler, TRUE);
    
    g_syncEvent = ::CreateEvent(NULL, FALSE, FALSE, NULL);

    rtsonlSetIniPath( iniPath );
    Identify( iniPath );
    rtsonlRegisterCallback( OnlineCallback );

    rtsonlOpen();

    ::WaitForSingleObject(g_syncEvent, 5000);

    // variant 1: show quotes
    char issue[256];
    ::GetPrivateProfileString("options", "issue_name", "", issue, sizeof(issue), iniPath);

    char minRec[256];
    minRec[0] = 0;
    if( issue[0] != 0 )
        sprintf(minRec, "%s" RTSONL_DELIMS " ", issue);
    char maxRec[256];
    maxRec[0] = 0;
    if( issue[0] != 0 )
        sprintf(maxRec, "%s" RTSONL_DELIMS "\x7F", issue);

/*
    RTSONL_CONNID connID = rtsonlOpenConnection( RTSONL_OPEN_DYNAMIC,
        "Quote", "moment,type,issue_name,firm_name,price,qty", "issue_name,firm_name", minRec, maxRec, 0 );
*/
    RTSONL_CONNID connID = rtsonlOpenConnection( RTSONL_OPEN_DYNAMIC,
        "Quote", "moment,type,issue_name,firm_name,price,qty", "rev", 0, 0, 0 );

/*
    // variant 2: show saved orders
    char firm[256];
    ::GetPrivateProfileString("options", "firm_name", "", firm, sizeof(firm), iniPath);

    char minRec[256];
    minRec[0] = 0;
    if( firm[0] != 0 )
        sprintf(minRec, "%s"RTSONL_DELIMS""RTSONL_DELIMS" ", firm);
    char maxRec[256];
    maxRec[0] = 0;
    if( firm[0] != 0 )
        sprintf(maxRec, "%s"RTSONL_DELIMS"\xFF"RTSONL_DELIMS"\xFF", firm);

    RTSONL_CONNID connID = rtsonlOpenConnection( RTSONL_OPEN_DYNAMIC,
        "SaveOrder", "moment,type,issue_name,firm_name,price,qty", "firm_name,issue_name,type", minRec, maxRec, 0 );
*/

    ::WaitForSingleObject(g_syncEvent, 30000);

    rtsonlPrintf( "********** OnlCon: press Enter to exit **********\n" );
    for( ; !g_stop; )
        {
        ::Sleep(100);
        }

    rtsonlCloseConnection( connID );
    rtsonlClose();
}

// EOF
