// RTS PLAZA
// Copyright (c) 1998, Technical Center of RTS
//
// RTS PLAZA Online: Display Database Scheme sample

#include <windows.h>
#include <stdio.h>

#include "Online/RTSOnl/Include/RTSOnl.h"

// Data
static BOOL     schemeReceived = FALSE;

// Sources

void DisplayScheme( const RTSONL_DB_SCHEME* scheme )
    {
    rtsonlPrintf( "------------------------------------------------------------------\n" );
    rtsonlPrintf( "-- RTS PLAZA Online Database Scheme\n" );
    rtsonlPrintf( "version:%d\n", scheme->version );
    rtsonlPrintf( "tables count:%d\n", scheme->tablesCount );
    
    // enumerate tables
    for( int tblnum = 0; tblnum < scheme->tablesCount; ++tblnum )
        {
        const RTSONL_TABLE* table = rtsonlGetTable( scheme, tblnum );
        if( ! table )
            {
            rtsonlPrintf( "-- scheme error: cannot get table %d\n", tblnum );
            break;
            }
        
        rtsonlPrintf( "  table %d: name:%s; columns:%d\n", 
            tblnum, table->name, table->columnsCount );

        for( int colnum = 0; colnum < table->columnsCount; ++colnum )
            {
            const RTSONL_COLUMN* column = rtsonlTableGetColumn( table, colnum );
            if( ! column )
                {
                rtsonlPrintf( "  -- scheme error: cannot get column %d in table %s\n",
                    colnum, table->name );
                break;
                }
            rtsonlPrintf( "    column %d: name:%s; format:%s\n",
                colnum, column->name, column->format );
            }
        }
    rtsonlPrintf( "-- scheme completed.\n" );
    }

void OnDSDetected()
    {
    rtsonlPrintf( "Scheme: DS Connected\n" );

    if( schemeReceived )
        return;

    if( !rtsonlOpenSchemeAsynch() )
        {
        rtsonlPrintf( "Scheme: cannot open database scheme\n" );
        return;
        }

    schemeReceived = TRUE;
    }

void CALLBACK OnlineCallback( UINT type, void** params )
    {
    switch( type )
        {
        case RTSONL_MSG_DS_CONNECTED:
            OnDSDetected();
            break;

        case RTSONL_MSG_DS_SCHEME: 
            {
            RTSONL_DB_SCHEME* scheme = (RTSONL_DB_SCHEME*)params[0];
            DisplayScheme(scheme);
            rtsonlCloseScheme(scheme);
            break;
            }

        default:
            break;
        }
    }

void main( int argc, char** argv )
    {
    rtsonlPrintf( 
        "RTS PLAZA Online. Scheme Sample Version 1.0\n"
        "Copyright (c) 1998, Technical Center of RTS\n" );

    char* iniPath = ".\\scheme.ini";
    if( argc > 1 )
        iniPath = argv[1];
    rtsonlPrintf( "Using ini path: '%s'\n", iniPath );

    SetConsoleOutputCP( GetACP() );

    rtsonlSetIniPath( iniPath );
    rtsonlRegisterCallback( OnlineCallback );

    rtsonlOpen();

    rtsonlPrintf( "********** Scheme: press Enter to exit **********\n" );
    for( ;; )
        {
        char    line[256];
        gets(line);
        if( ! *line )
            break;
        }

    rtsonlClose();
    }
