// RTS PLAZA
// Copyright (c) 2000, Technical Center of RTS
//
// RTS PLAZA Online: Send Message sample

#include <windows.h>
#include <stdio.h>

#include "Online/RTSOnl/Include/RTSOnl.h"


/////////////////////////////////////////////////////////////////////////////
// GLOBALS 
//----------------------------------------------------------  --//
// find pointer on the file into command line [argv]
const char* find_filename_in_commandline (int argc, char** argv)
{
    if (argc && argv)
    {
        while (--argc)
        {
            char *pCur = *++argv;
            if ( (pCur) && (*pCur) && (*pCur != '/') && (*pCur != '-') )
            {
                return pCur;
            }
        } 
    }

    return NULL;
}
//----------------------------------------------------------  --//
// find pointer on the parameter [parm_name] into command line [argv]
const char* find_parm_in_commandline (int argc, char** argv, const char* parm_name)
{
    int parm_len = strlen (parm_name);

    if (argc && argv)
    {
        while (--argc)
        {
            char *pCur = *++argv;
            if ( (pCur) && ( (*pCur == '/') || (*pCur == '-') ) )
            {
                if (strncmp (parm_name, ++pCur, parm_len) == 0)
                {
                    switch (pCur[parm_len])
                    {
                        case '\0': return pCur + parm_len;
                        case '=':  return pCur + parm_len + 1;
                    }
                }
            }
        }
    }

    return NULL;
}

/////////////////////////////////////////////////////////////////////////////
// additional functions
//----------------------------------------------------------  --//
void DisplayMsgScheme (const RTSONL_MSG_SCHEME* pScheme)
{
    rtsonlPrintf( "------------------------------------------------------------------\n" );
    rtsonlPrintf( "-- RTS PLAZA Online Messages Scheme\t version:%d\t messages count:%d\n",
        (pScheme) ? pScheme->version       : 0,
        (pScheme) ? pScheme->messagesCount : 0 );
    
    if (pScheme)
    {
        // enumerate messages
        for (int iMsgNum = 0; iMsgNum != pScheme->messagesCount; ++iMsgNum)
        {
            const RTSONL_MSG* pMsgHeader = rtsonlGetMsg (pScheme, iMsgNum);
            if (!pMsgHeader)
            {
                rtsonlPrintf ("! Error: cannot get message #%d definition\n", iMsgNum);
                break;
            }
        
            rtsonlPrintf ("  msg %d:  name:%s  fields:%d\n", 
                iMsgNum, pMsgHeader->name, pMsgHeader->fieldsCount );

            rtsonlPrintf ("     field                              name   format\n");

            for (int iFieldNum = 0; iFieldNum != pMsgHeader->fieldsCount; ++iFieldNum)
            {
                const RTSONL_MSG_FIELD* pField = rtsonlGetMsgField (pMsgHeader, iFieldNum);
                if (!pField)
                {
                    rtsonlPrintf ("! Error: cannot get field #%d in message %s\n",
                        iFieldNum, pMsgHeader->name);
                    break;
                }
                rtsonlPrintf ("       %3d  %32s %8s\n",
                    iFieldNum, pField->name, pField->format);
            }
        }
    }
    rtsonlPrintf( "-- scheme completed.\n" );
}
//----------------------------------------------------------  --//
bool SendMessage
    (const RTSONL_MSG_SCHEME* pScheme,
     const char* pMsgName,
     const char* pMsgFields,
     const char* pMsgValues)
{
    rtsonlPrintf ("Send: #name=%s\n\t#fields=%s\n\t#values=%s\n",
        pMsgName, pMsgFields, pMsgValues);

    if (rtsonlSendMsg (pScheme, pMsgName, pMsgFields, pMsgValues))
    {
        if (const char* pszReplyName = rtsonlGetReplyName())
        {
            rtsonlPrintf ("Send OK, Reply: #name=%s\n\t", pszReplyName);

            // Finding reply message description
            if (const RTSONL_MSG* pMsgHeader = rtsonlFindMsg (pScheme, pszReplyName))
            {
                // Output filds list
                rtsonlPrintf ("#fields=");
                int iFieldNum;
                for (iFieldNum = 0; iFieldNum != pMsgHeader->fieldsCount; ++iFieldNum)
                {
                    if (const RTSONL_MSG_FIELD* pField = rtsonlGetMsgField (pMsgHeader, iFieldNum))
                        rtsonlPrintf ( (iFieldNum) ? ",%s" : "%s", pField->name);
                }

                // Output values list
                rtsonlPrintf ("\n\t#values=");
            
                for (iFieldNum = 0; iFieldNum != pMsgHeader->fieldsCount; ++iFieldNum)
                {
                    if (const RTSONL_MSG_FIELD* pField = rtsonlGetMsgField (pMsgHeader, iFieldNum))
                        rtsonlPrintf ( (iFieldNum) ? ",%s" : "%s", rtsonlGetReplyValue(pField->name));
                }
            }

            rtsonlPrintf ("\n");

            return true;
        }
        else
            rtsonlPrintf ("! Error: reply is empty\n");
    }
    else
        rtsonlPrintf ("! Error: %s\n", rtsonlGetReplyError());

    return false;
}

//----------------------------------------------------------  --//
void CALLBACK OnlineCallback (UINT type, void** params)
{
}
//----------------------------------------------------------  --//
bool Identify
    (const char* pszIniFile,
     const char* pszUser,
     const char* pszPWD,
     const char* pszPWDExtra)
{
    char xUser[64], xPWD[64], xPWDExtra[64];

    #define myCheckStr(pStr,pExtBuffer,pKey,pDefault)\
        if (!pStr)\
          GetPrivateProfileString("SendMsg", pKey, pDefault, (char*)(pStr = pExtBuffer), sizeof(pExtBuffer), pszIniFile);

    myCheckStr (pszUser,     xUser,     "NAME",          "")
    myCheckStr (pszPWD,      xPWD,      "PASSWORD",      "")
    myCheckStr (pszPWDExtra, xPWDExtra, "EXTRAPASSWORD", "GUI")

    #undef myCheckStr

    if (*pszUser)
    {
        if (!rtsonlIdentify (pszUser, pszPWD, pszPWDExtra))
        {
            rtsonlPrintf ("! Error opening connection: Authorization Required\n");
            return false;
        }
        rtsonlPrintf ("pening connection, user: '%s'\n", pszUser);
    }
    return true;
}

/////////////////////////////////////////////////////////////////////////////
// main
//----------------------------------------------------------  --//
void main (int argc, char* argv[])
{
    //SetConsoleOutputCP (GetACP());
    //setlocale (LC_CTYPE, "russian");

    const char* pszIniFile = (argc > 1) ? find_filename_in_commandline (2, argv) : NULL;
    if (!pszIniFile)
        pszIniFile = "SendMsg.ini";

    //-- Init RTSOnline
    rtsonlSetIniPath (pszIniFile);

    //-- Hello
    rtsonlPrintf ("\n\n\
    RTS PLAZA Online. Send Message sample Version 1.0\n\
    Copyright (c) 2000, Technical Center of RTS\n\
    \n\
    Usage: SendMsg <ini-file> [options]\n\
        options:\n\
            identify options:\n\
        -user=<username> - user in Trading System in form 'FIRM@WKS'\n\
        -pwd=<password>  - password in Trading System\n\
            crypto options:\n\
        -keydir=<keydir> - directory with key diskette\n\
        -certid=<certid> - ID of user certificate\n\
        -from=<from>     - user e-mail address\n");

    if (*pszIniFile)
    {
        rtsonlPrintf ("SendMsg started with ini-file: '%s'\n", pszIniFile);

        if (Identify (pszIniFile,
                  find_parm_in_commandline (argc, argv, "user"),
                  find_parm_in_commandline (argc, argv, "pwd"),
                  NULL))
        {
            rtsonlRegisterCallback (OnlineCallback);

            //---Run RTSOnline
            rtsonlOpen();

            Sleep (1000);

            if (const RTSONL_MSG_SCHEME* pScheme = rtsonlOpenMsgScheme())
            {
                if ( !find_parm_in_commandline (argc, argv, "keydir") ||
                     rtsonlInitCryptoSign (find_parm_in_commandline (argc, argv, "keydir"),
                                           find_parm_in_commandline (argc, argv, "certid"),
                                           find_parm_in_commandline (argc, argv, "from")))
                {
                    //-- simple test
                    DisplayMsgScheme (pScheme);

                    /*
                    SendMessage (pScheme, "MsgNews",
                        "subject, body, msg_action, source_id,  keywords",
                        "WWW1, test body3, N, RTS, keyword1_keyword3");

                    /*
                    SendMessage (pScheme, "MsgOrder",
                        "type,issue_name ,issueID, firm_name, firmID,msg_action, qty, price, paycond",
                        "A, TEST4,283, ADMIN,1,N, 10000L, 1.23, B");
                    */
                    SendMessage (pScheme, "MsgOrder",
                        "catalyst, firmID, firm_name, i_code, issueID, issue_name, order_type, paycond, price, qty, quote_order, settl_pair, type",
                        " 1, 503, TRN2M, RF0000000005, 450, EESRG, 1, B, 3.6L, 100L, 1, trn2m1, A");

                    rtsonlCloseCryptoSign ();
                }

                rtsonlCloseMsgScheme (pScheme);
            }
            else
            {
                rtsonlPrintf( "! Error: cannot asynch scheme open\n" );
            }

            //-- Close RTSOnline
            rtsonlClose();
        }
    }
    else
        rtsonlPrintf ("! Error: Need ini-file.\n");


    rtsonlPrintf ("********** press Enter to exit **********\n");
    char line[256];
    do {gets(line);} while (*line);
}


// EOF