// RTS PLAZA
// Copyright (c) 2000, Technical Center of RTS
//
// RTS PLAZA Online: Batch - file executor for message sending

#include <windows.h>
#include <stdio.h>

#include "Online/RTSOnl/Include/RTSOnl.h"


/////////////////////////////////////////////////////////////////////////////
// GLOBALS 
//----------------------------------------------------------  --//
// find pointer on the file into command line [argv]
const char* find_filename_in_commandline (int argc, char** argv)
{
    if (argc && argv)
    {
        while (--argc)
        {
            char *pCur = *++argv;
            if ( (pCur) && (*pCur) && (*pCur != '/') && (*pCur != '-') )
            {
                return pCur;
            }
        } 
    }

    return NULL;
}
//----------------------------------------------------------  --//
// find pointer on the parameter [parm_name] into command line [argv]
const char* find_parm_in_commandline (int argc, char** argv, const char* parm_name)
{
    int parm_len = strlen (parm_name);

    if (argc && argv)
    {
        while (--argc)
        {
            char *pCur = *++argv;
            if ( (pCur) && ( (*pCur == '/') || (*pCur == '-') ) )
            {
                if (strncmp (parm_name, ++pCur, parm_len) == 0)
                {
                    switch (pCur[parm_len])
                    {
                        case '\0': return pCur + parm_len;
                        case '=':  return pCur + parm_len + 1;
                    }
                }
            }
        }
    }

    return NULL;
}

/////////////////////////////////////////////////////////////////
// TStringTranslator
//
struct TStringTranslator
{
    //   pStr   : <wide_case>[<wide_case>]
    // -: true - pStr   pszMatrix.
    static bool CompareWithKeyword  (const char *pStr,  const char *pszMatrix);

    //   pStr  
    // -: true - pStr   pszMatrix.
    static bool CompareWithKeywordCS  (const char *pStr,  const char *pszMatrix);

    //   pStr    :
    //    <wide_case>[<wide_case>] ' '
    //  <wide_case> = <uppercase><lowercase> | <lowercase><uppercase>
    // -:  ( 0) - pStr     pszMatrix.
    //        INT_MAX       -.
    static int CompareWithKeywords  (const char *pStr,  const char *pszMatrix, const char **ppEnd = NULL);

    //   pStr    :
    //    <> ' '
    // -:  ( 0) - pStr     pszMatrix.
    //        0xffff       -.
    static int CompareWithKeywordsCS  (const char *pStr,  const char *pszMatrix);

    //  : , ,  ,  ,
    //    (: /*...*/    //...\n)
    static char* GotoRealSymbol (const char *pStr);

    //   : , ,  ,  ,
    static char* ReturnToRealSymbol (const char *pStr);

    //   (  : /*...*/    //...\n)
    // -: null-
    static char* PassCommentary (const char *pStr);  //-    '/'.

    //   (    "")
    // -: null-
    static char* PassQuotedString (const char *pStr);  //-    '"'.

    //   ,    [pStr],    #1 [pBaseStr].
    static int GetStringNumber (const char *pBaseStr, const char *pStr);

    //
    static inline bool IsDelimiter (const char cSymbol)
    {
        switch (cSymbol)
        {
          case '\0':  case '\t':  case '\n':  case '\r':  case ' ':  return true;
        }
        return false;
    }
};
//////////////////////////////////////////////////////{+++ TStringTranslator +++}
//----------------------------------------------------------------  ----//
bool TStringTranslator::CompareWithKeyword
  (const char *pStr,
   const char *pszMatrix)
{
  while (*pszMatrix)
  {
    if ( (*pStr != *(pszMatrix++)) &&
         (*pStr != *pszMatrix)      ) return false;
    pStr++;
    pszMatrix++;
  }
  return true;
}
//----------------------------------------------------------------  ----//
bool TStringTranslator::CompareWithKeywordCS
  (const char *pStr,
   const char *pszMatrix)
{
  while (*pszMatrix)
  {
    if (*pStr != *pszMatrix)  return false;
    pStr++;
    pszMatrix++;
  }
  return true;
}
//----------------------------------------------------------------  ----//
int TStringTranslator::CompareWithKeywords
  (const char *pStr,
   const char *pszMatrix,
   const char **ppEnd)
{
  int iResult = 0;
  const char *pCurStr = pStr;

  while ( (*pszMatrix != '\0') && (*pszMatrix != ' ') )
  {
    if ( (*pCurStr != *(pszMatrix++)) &&
         (*pCurStr != *pszMatrix)      )
    { //--     .
      while ( (*pszMatrix != '\0') && (*(pszMatrix++) != ' ') ) {};
      pCurStr = pStr;
      iResult++;
    }
    else
    {
      pCurStr++;
      pszMatrix++;
    }
  }
  if ( (*pszMatrix == '\0') && (pCurStr == pStr) ) return 0xffff;
  if (ppEnd)  *ppEnd = pCurStr;
  return iResult;
}
//----------------------------------------------------------------  ----//
int TStringTranslator::CompareWithKeywordsCS
  (const char *pStr,
   const char *pszMatrix)
{
  int iResult = 0;
  const char *pCurStr = pStr;

  while ( (*pszMatrix != '\0') && (*pszMatrix != ' ') )
  {
    if (*pCurStr != *pszMatrix)
    { //--     .
      while ( (*pszMatrix != '\0') && (*(pszMatrix++) != ' ') ) {};
      pCurStr = pStr;
      iResult++;
    }
    else
    {
      pCurStr++;
      pszMatrix++;
    }
  }
  if ( (*pszMatrix == '\0') && (pCurStr == pStr) ) return 0xffff;
  return iResult;
}
//----------------------------------------------------------------  ----//
char* TStringTranslator::GotoRealSymbol (const char *pStr)
{
  while (true)
  {
    switch (*pStr)
    {
      case ' ' :          //-> 
      case '\t':          //-> 
      case '\n':          //->  
      case '\r':          //->  
        pStr++;
        continue;

      case '/':          //-> / 
        switch (*(++pStr))
        {
          case '*':
            while ( (*pStr) &&
                    ( (*(++pStr) != '*') || (*(++pStr) != '/') ) )
            {}
            if (*pStr) pStr++;
            break;

          case '/':
            while ( (*(++pStr)) && (*pStr != '\n') )
            {}
            if (*pStr) pStr++;
            break;

          default:
            return (char*)(pStr-1);
        }
        continue;
    }
    return (char*)pStr;
  }
}
//----------------------------------------------------------------  ----//
char* TStringTranslator::ReturnToRealSymbol (const char *pStr)
{
  while (true)
  {
    switch (*pStr)
    {
      case ' ' :          //-> 
      case '\t':          //-> 
      case '\n':          //->  
      case '\r':          //->  
        pStr--;
        continue;
    }
    return (char*)pStr;
  }
}
//----------------------------------------------------------------  ----//
char* TStringTranslator::PassCommentary (const char *pStr)
{
  if (*pStr == '/')          //-> / 
  {
    switch (*(++pStr))
    {
      case '*':
        while ( (*pStr) &&
                ( (*(++pStr) != '*') || (*(++pStr) != '/') ) )
        {}
        pStr = (*pStr) ? pStr+1 : NULL;
        break;

      case '/':
        while ( (*(++pStr)) && (*pStr != '\n') )
        {}
        if (*pStr) {if (*(++pStr) == '\r')  pStr++;}
        break;

      default:
        pStr--;
        break;
    }
  }
  return (char*)pStr;
}
//----------------------------------------------------------------  ----//
char* TStringTranslator::PassQuotedString (const char *pStr)
{
  if (*pStr == '"')          //-> 
  {
  	//   '"'
    while (true)
    {
        switch (*(++pStr))
        {
          case '\0':  pStr = NULL; break;
          case '\n': if (*(pStr-1) != '\\')  {pStr = NULL; break;}  else  continue;
          case '"' : if (*(pStr-1) != '\\')  {pStr++; break;}
          default:  continue;
        }
        break;
    }
  }
  return (char*)pStr;
}
//----------------------------------------------------------------  ----//
int TStringTranslator::GetStringNumber (const char *pBaseStr, const char *pStr)
{
    int iRes = 1;
    while (pBaseStr != pStr) {if (*pBaseStr++ == '\n')  iRes++;}
    return iRes;
}

/////////////////////////////////////////////////////////////////////////////
// CFileExecutor
//
class CFileExecutor
{
    private:
        bool           m_bRun;  // true -run
        char           m_ErrorStr[256];

        const RTSONL_MSG_SCHEME *m_pScheme;

        //    
        HANDLE m_hFile;     //  
        HANDLE m_hMMFFile;  //  
        void       *m_pMMFFile;
        const char *m_pPosition;    //   
        int         m_iNumString;   //    

    private:
        bool fn_ActivateMessage ();
        bool fn_SendMessage
            (const RTSONL_MSG_SCHEME* pScheme,
             const char* pMsgName,
             const char* pMsgFields,
             const char* pMsgValues);

    public:
        CFileExecutor (const RTSONL_MSG_SCHEME* pScheme, const char* pFileName);
        ~CFileExecutor ();

        bool Run ();
        void Pause ()   {m_bRun = false;};

        bool IsPaused() {return !m_bRun;};

        const char* GetErrorStr () {return m_ErrorStr;};
};

/////////////////////////////////////////////////////////////////////////////
// CFileExecutor
//----------------------------------------------------------  --//
CFileExecutor::CFileExecutor (const RTSONL_MSG_SCHEME* pScheme, const char* pFileName) :
    m_bRun  (false),
    m_hFile (INVALID_HANDLE_VALUE),
    m_pScheme (pScheme),
    m_hMMFFile (0),
    m_pMMFFile (NULL),
    m_pPosition (NULL),
    m_iNumString (0)
{
    *m_ErrorStr = '\0';
	// Open file
	m_hFile = CreateFile (pFileName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);

	if (m_hFile != INVALID_HANDLE_VALUE)
	{
        try
        {
            if (DWORD dwSize = GetFileSize(m_hFile, NULL))
            {
	            if (m_hMMFFile = CreateFileMapping (m_hFile, NULL, PAGE_READONLY, 0, 0, NULL))
	            {
	                if (m_pMMFFile = MapViewOfFile (m_hMMFFile, FILE_MAP_READ, 0, 0, 0))
                    {
                        if (*((char*)m_pMMFFile + dwSize) == '\0')
                        {
	                        //    
	                        m_pPosition  = (const char*)m_pMMFFile;
	                        m_iNumString = 1;
                            return;
                        }
                        else
	                    {
                            _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "File \'%s\' is not in text format (need ending 0)\n", pFileName);
	                    }
                        UnmapViewOfFile (m_pMMFFile);
	                }
  	                else
	                {
		                _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Impossible to map file \'%s\'\n", pFileName);
	                }
       	            CloseHandle (m_hMMFFile);
                }
  	            else
	            {
		            _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Impossible create map file \'%s\'\n", pFileName);
	            }
            }
  	        else
	        {
		        _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Impossible to run empty file \'%s\'\n", pFileName);
	        }
        }
        catch (...)
        {
       	    CloseHandle (m_hFile);
            throw;
        }
       	CloseHandle (m_hFile);
        m_hFile = INVALID_HANDLE_VALUE;
    }
    else
    {
        _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Impossible to open file \'%s\'\n", pFileName);
    }
}
//----------------------------------------------------------  --//
CFileExecutor::~CFileExecutor ()
{
    if (m_hFile != INVALID_HANDLE_VALUE)
    {
        // free file
        UnmapViewOfFile (m_pMMFFile);
        CloseHandle     (m_hMMFFile);
        CloseHandle     (m_hFile);
    }
}
//----------------------------------------------------------  --//
bool CFileExecutor::fn_ActivateMessage ()
{
    bool bRes = false;
    char *pName   = TStringTranslator::GotoRealSymbol (m_pPosition);
    char *pFields = pName;

    while (*pFields && (*pFields != '(') )  {pFields++;}
    
    int iLen = TStringTranslator::ReturnToRealSymbol (pFields-1) - pName;
    char *pRealName = new char[(++iLen)+1];
    memcpy (pRealName, pName, iLen);
    pRealName[iLen] = '\0';
    
    pFields++;
    char *pValues = pFields;
    while (*pValues && (*pValues != ')') )  {pValues++;}

    iLen = pValues - pFields;
    char *pRealFields = new char[iLen+1];
    memcpy (pRealFields, pFields, iLen);
    pRealFields[iLen] = '\0';

    pValues = TStringTranslator::GotoRealSymbol (pValues+1);

    if (*pValues == '=')
    {
        pValues = TStringTranslator::GotoRealSymbol (pValues+1);

        if (*pValues == '(')
        {
            char *pEndOfValues = ++pValues;
            while (*++pEndOfValues && ( (*pEndOfValues != ')') || (*(pEndOfValues-1) == '\\') ) )  {}

            iLen = pEndOfValues - pValues;
            char *pRealValues = new char[iLen+1];
            memcpy (pRealValues, pValues, iLen);
            pRealValues[iLen] = '\0';

            if (*(m_pPosition = pEndOfValues))
                m_pPosition = TStringTranslator::GotoRealSymbol (m_pPosition+1);

            if (*pRealName && *pRealFields && *pRealValues)
            {
                if (fn_SendMessage (m_pScheme, pRealName,
                        pRealFields,
                        pRealValues))
                    bRes = true;
            }
            else
                _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Syntax error");

            delete[] pRealValues;
        }
        else
            _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Expected symbol '(' after '='");
    }
    else
        _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Expected symbol '=' after ')'");

    delete[] pRealFields;

    delete[] pRealName;

    return bRes;
}
//----------------------------------------------------------  --//
bool CFileExecutor::fn_SendMessage
    (const RTSONL_MSG_SCHEME* pScheme,
     const char* pMsgName,
     const char* pMsgFields,
     const char* pMsgValues)
{
    #ifdef _DEBUG
    _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Send: #name=%s\n\t#fields=%s\n\t#values=%s\n",
        pMsgName, pMsgFields, pMsgValues);
    #endif

    if (rtsonlSendMsg (pScheme, pMsgName, pMsgFields, pMsgValues))
    {
        if (const char* pszReplyName = rtsonlGetReplyName())
        {
            #ifdef _DEBUG
            int iLen = _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Send OK, Reply: #name=%s\n", pszReplyName);

            // Finding reply message description
            if (const RTSONL_MSG* pMsgHeader = rtsonlFindMsg (pScheme, pszReplyName))
            {
                // Output filds list
                int iDelta = _snprintf (m_ErrorStr + iLen, sizeof(m_ErrorStr) - iLen, "\t#fields=");
                if (iDelta > 0)
                {
                    iLen+= iDelta;
                    int iFieldNum;
                    for (iFieldNum = 0; iFieldNum != pMsgHeader->fieldsCount; ++iFieldNum)
                    {
                        if (const RTSONL_MSG_FIELD* pField = rtsonlGetMsgField (pMsgHeader, iFieldNum))
                        {
                            iDelta = _snprintf (m_ErrorStr + iLen, sizeof(m_ErrorStr) - iLen, "%s,", pField->name);
                            if (iDelta < 0)  break;
                            iLen+= iDelta;
                        }
                    }

                    // Output values list
                    iLen--;
                    iDelta = _snprintf (m_ErrorStr + iLen, sizeof(m_ErrorStr) - iLen, " \n\t#values=");
            
                    if (iDelta > 0)
                    {
                        iLen+= iDelta;
                        for (iFieldNum = 0; iFieldNum != pMsgHeader->fieldsCount; ++iFieldNum)
                        {
                            if (const RTSONL_MSG_FIELD* pField = rtsonlGetMsgField (pMsgHeader, iFieldNum))
                            {
                                iDelta = _snprintf (m_ErrorStr + iLen, sizeof(m_ErrorStr) - iLen, "%s,", rtsonlGetReplyValue(pField->name));
                                if (iDelta < 0)  break;
                                iLen+= iDelta;
                            }
                        }
                    }
                    m_ErrorStr[iLen-1] = '\n';
                    m_ErrorStr[iLen]   = '\0';
                }
            }

            #endif  //  _DEBUG

            return true;
        }
        else
            _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Send error: reply is empty\n");
    }
    else
        _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Send error: %s\n", rtsonlGetReplyError());

    return false;
}


static int    theArgc;
static char **theArgv;


#pragma warning(disable:4786)
#include <map>
typedef std::map <int,long> theLoopsList_t;
theLoopsList_t theLoopsList;

HANDLE theEvent;
//----------------------------------------------------------  --//
BOOL WINAPI ctrlHandler (DWORD e)
{
    SetEvent (theEvent);
    return FALSE;
}

//----------------------------------------------------------  --//
bool CFileExecutor::Run ()
{
    if (m_hFile != INVALID_HANDLE_VALUE)
    {
        m_bRun = true;
        
        //-- RUN
        while ( m_bRun && *(m_pPosition = TStringTranslator::GotoRealSymbol (m_pPosition)) )
        {
            {   // console output current line
                char *pDst = m_ErrorStr + sprintf (m_ErrorStr, "<%2d> ", TStringTranslator::GetStringNumber((const char*)m_pMMFFile, m_pPosition));

                const char *pCur = m_pPosition;
                while (*pCur && (*pCur != '\n')) {pCur++;}
                int iLen = pCur - m_pPosition;
                if (iLen >= sizeof(m_ErrorStr) - (pDst - m_ErrorStr) - 1)  iLen = sizeof(m_ErrorStr) - (pDst - m_ErrorStr) - 2;
                memcpy (pDst, m_pPosition, iLen);
                pDst[iLen]   = '\n';
                pDst[iLen+1] = '\0';

                rtsonlPrintf (m_ErrorStr);
                
                *m_ErrorStr = '\0';
            }

            switch (TStringTranslator::CompareWithKeywords (m_pPosition,
                "sSeEnNdD dDeElLaAyY lLoOoOpPsS eExXiItT "
                "iInNiItTcCrRyYpPtToOsSiIgGnN cClLoOsSeEcCrRyYpPtToOsSiIgGnN", &m_pPosition))
            {
                case 0: //-> send
                    if (TStringTranslator::IsDelimiter(*m_pPosition))
                    {
                        m_pPosition++;
                        if (!fn_ActivateMessage ())
                        {
                            m_bRun = false;
                            return false;
                        }
                    }
                    else
                    {
                        _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Expected space symbol after operator: send");
                        m_bRun = false;
                        return false;
                    }
                    break;

                case 1: //-> delay
                    if (TStringTranslator::IsDelimiter(*m_pPosition))
                    {
                        DWORD dwTimeOut = strtoul (m_pPosition+1, (char**)&m_pPosition, 10);
                        if (TStringTranslator::IsDelimiter(*m_pPosition) && (dwTimeOut < 3600000))
                        {
                            if (theEvent = CreateEvent (NULL, FALSE, FALSE, NULL))
                            {
                                // add handler
                                SetConsoleCtrlHandler (ctrlHandler, TRUE);

                                // delay
                                WaitForSingleObject (theEvent, dwTimeOut);

                                // remove handler
                                SetConsoleCtrlHandler (ctrlHandler, FALSE);
                                CloseHandle (theEvent);
                            }
                        }
                        else
                        {
                            _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Expected number symbols after operator: delay");
                            m_bRun = false;
                            return false;
                        }
                    }
                    else
                    {
                        _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Expected space symbol after operator: delay");
                        m_bRun = false;
                        return false;
                    }
                    break;

                case 2: //-> loops
                    if (TStringTranslator::IsDelimiter(*m_pPosition))
                    {
                        int iLoopId = m_pPosition - (const char*)m_pMMFFile;

                        DWORD dwLoopsCount = strtoul (m_pPosition+1, (char**)&m_pPosition, 10);

                        if (TStringTranslator::IsDelimiter(*m_pPosition))
                        {
                            theLoopsList_t::iterator iter = theLoopsList.find (iLoopId);

                            if (iter != theLoopsList.end())
                            {
                                if ( iter->second && !--(iter->second))
                                {
                                    theLoopsList.erase (iter);
                                    continue;
                                }
                            }
                            else
                                theLoopsList.insert (theLoopsList_t::value_type(iLoopId, dwLoopsCount));

                            m_pPosition  = (const char*)m_pMMFFile;
                        }
                        else
                        {
                            _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Expected number symbols after operator: loops");
                            m_bRun = false;
                            return false;
                        }
                    }
                    else
                    {
                        _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Expected space symbol after operator: delay");
                        m_bRun = false;
                        return false;
                    }
                    break;

                case 3: //-> exit
                    if (TStringTranslator::IsDelimiter(*m_pPosition))
                    {
                        m_bRun = false;
                    }
                    else
                    {
                        _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Expected space symbol after operator: exit");
                        m_bRun = false;
                        return false;
                    }
                    break;

                case 4: //-> InitCryptoSign
                    if (TStringTranslator::IsDelimiter(*m_pPosition))
                    {
                        m_pPosition++;

                        if (find_parm_in_commandline (theArgc, theArgv, "keydir"))
                        {
                            rtsonlInitCryptoSign (find_parm_in_commandline (theArgc, theArgv, "keydir"),
                                                  find_parm_in_commandline (theArgc, theArgv, "certid"),
                                                  find_parm_in_commandline (theArgc, theArgv, "from"));
                        }
                        else
                        {
                            _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "need command line parameters for init crypto");
                            m_bRun = false;
                            return false;
                        }

                    }
                    else
                    {
                        _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Expected space symbol after operator: InitCryptoSign");
                        m_bRun = false;
                        return false;
                    }
                    break;

                case 5: //-> CloseCryptoSign
                    if (TStringTranslator::IsDelimiter(*m_pPosition))
                    {
                        m_pPosition++;
                        rtsonlCloseCryptoSign ();
                    }
                    else
                    {
                        _snprintf (m_ErrorStr, sizeof(m_ErrorStr), "Expected space symbol after operator: CloseCryptoSign");
                        m_bRun = false;
                        return false;
                    }
                    break;



                default:
                    {
                        strcpy (m_ErrorStr, "Unknown operator: ");

                        // adding "operator" name
                        const char *pCur = m_pPosition;
                        while (!TStringTranslator::IsDelimiter(*++pCur)) {}
                        size_t iLen   = pCur - m_pPosition;
                        size_t iUse = strlen(m_ErrorStr);
                        pCur = m_ErrorStr + iUse;
                        if (iLen >= sizeof(m_ErrorStr) - iUse)  iLen = sizeof(m_ErrorStr) - iUse - 1;
                        memcpy ((char*)pCur, m_pPosition, iLen);
                        ((char*)pCur)[iLen] = '\0';

                        m_bRun = false;
                        return false;
                    }
            }
        }
        return true;
    }
    return false;
}

/////////////////////////////////////////////////////////////////////////////
// additional functions
//
//----------------------------------------------------------  --//
void CALLBACK OnlineCallback (UINT type, void** params)
{
}
//----------------------------------------------------------  --//
bool Identify
    (const char* pszIniFile,
     const char* pszUser,
     const char* pszPWD,
     const char* pszPWDExtra)
{
    char xUser[64], xPWD[64], xPWDExtra[64];

    //          
    //    rtsonlIdentify (  ,    ).

    #define myCheckStr(pStr,pExtBuffer,pKey,pDefault)\
        if (!pStr)\
          GetPrivateProfileString("SendMsg_bat", pKey, pDefault, (char*)(pStr = pExtBuffer), sizeof(pExtBuffer), pszIniFile);

    myCheckStr (pszUser,     xUser,     "NAME",          "")
    myCheckStr (pszPWD,      xPWD,      "PASSWORD",      "")
    myCheckStr (pszPWDExtra, xPWDExtra, "EXTRAPASSWORD", "GUI")

    #undef myCheckStr

    if (*pszUser)
    {
        if (!rtsonlIdentify (pszUser, pszPWD, pszPWDExtra))
        {
            rtsonlPrintf ("! Error opening connection: Authorization Required\n");
            return false;
        }
        rtsonlPrintf ("pening connection, user: '%s'\n", pszUser);
    }
    return true;
}


/////////////////////////////////////////////////////////////////////////////
// main
//----------------------------------------------------------  --//
void main (int argc, char* argv[])
{
    theArgc = argc;
    theArgv = argv;
    //SetConsoleOutputCP (GetACP());
    //setlocale (LC_CTYPE, "russian");

    const char* pszIniFile = (argc > 1) ? find_filename_in_commandline (2, argv) : NULL;
    if (!pszIniFile)
        pszIniFile = "SendMsg_bat.ini";

    //-- Init RTSOnline
    rtsonlSetIniPath (pszIniFile);

    //-- Hello
    rtsonlPrintf ("\n\n\
    RTS PLAZA Online. Batch-file executor for message sending. Version 1.0\n\
    Copyright (c) 2000, Technical Center of RTS\n\
    \n\
    Usage: SendMsg_bat <ini-file> [options] <batch-file>\n\
        options:\n\
            identify options:\n\
        -user=<username> - user in Trading System in form 'FIRM@WKS'\n\
        -pwd=<password>  - password in Trading System\n\
            crypto options:\n\
        -keydir=<keydir> - directory with key diskette\n\
        -certid=<certid> - ID of user certificate\n\
        -from=<from>     - user e-mail address\n");

    if (*pszIniFile)
    {
        rtsonlPrintf ("SendMsg_bat started with ini-file: '%s'\n", pszIniFile);

        if (Identify (pszIniFile,
                  find_parm_in_commandline (argc, argv, "user"),
                  find_parm_in_commandline (argc, argv, "pwd"),
                  NULL))
        {
            rtsonlRegisterCallback (OnlineCallback);

            //---Run RTSOnline
            rtsonlOpen();

            Sleep (1000);

            if (const RTSONL_MSG_SCHEME* pScheme = rtsonlOpenMsgScheme())
            {
                if ( !find_parm_in_commandline (argc, argv, "keydir") ||
                     rtsonlInitCryptoSign (find_parm_in_commandline (argc, argv, "keydir"),
                                           find_parm_in_commandline (argc, argv, "certid"),
                                           find_parm_in_commandline (argc, argv, "from")))
                {
                    const char* pszBatFile = (argc > 1) ? find_filename_in_commandline (argc-1, argv+1) : NULL;
                    if (!pszBatFile)
                        pszBatFile = "SendMsg_bat.txt";

                    if (*pszBatFile)
                    {
                        rtsonlPrintf ("********** RUN bat-file: '%s'. press Ctrl+C to pause or exit **********\n", pszBatFile);

                        //-- run bat-file as test
                   	    CFileExecutor* pExec = new CFileExecutor(pScheme, pszBatFile);

                        try
                        {
                            while (!pExec->Run ())
                            {
                                rtsonlPrintf ("! %s\n", pExec->GetErrorStr());
                                int iCont;
                                do
                                {
                                    rtsonlPrintf ("********** PAUSE. enter 'Y' to continue or 'N' to exit **********\n");
                                    iCont = getchar ();
                                    if      (iCont == 'Y')  iCont = 'y';
                                    else if (iCont == 'N')  iCont = 'n';
                                } while ( (iCont != 'y') && (iCont != 'n') );
                                
                                while (getchar () != '\n')  {};
                                
                                if (iCont == 'n')  break;
                            }



                            /*
                                MSG msg;
                                while (GetMessage(&msg, NULL, 0, 0))
                                {
                                    if (msg.message == WM_KEYDOWN)
                                    {
                                        if (msg.wParam == VK_RETURN)
                                            break;

                                        if (msg.wParam == VK_SPACE)
                                        {
                                            // Pause running
                                            pExec->Pause();
                                            rtsonlPrintf ("********** PAUSE. press any key to continue **********\n");
                                        }
                                        else if (pExec->IsPaused())
                                        {
                                            rtsonlPrintf ("********** RUN. press Space to pause or Enter to exit **********\n");
                                            pExec->Run();
                                        }
                                    }
                                    TranslateMessage(&msg);
                                    DispatchMessage(&msg);
                                }
                            */
                        }
                        catch (...)
                        {
                            delete pExec;
                            throw;
                        }

                        // Stop running
                        delete pExec;
                    }
                    else
                        rtsonlPrintf( "! Error: Need batch-file.\n" );

                    rtsonlCloseCryptoSign ();
                }

                rtsonlCloseMsgScheme (pScheme);
            }
            else
                rtsonlPrintf( "! Error: cannot asynch open message scheme\n" );

            //-- Close RTSOnline
            rtsonlClose();
        }
    }
    else
        rtsonlPrintf ("! Error: Need ini-file.\n");


    rtsonlPrintf ("********** press Enter to exit **********\n");
    char line[256];
    do {gets(line);} while (*line);
}


// EOF